@@ -1,4 +1,4 @@
-use {TryRead, TryWrite, Result, check_len};
+use {check_len, Result, TryRead, TryWrite};
 
 impl<'a> TryRead<'a> for bool {
     #[inline]
@@ -18,4 +18,4 @@ impl TryWrite for bool {
 
         Ok(1)
     }
-}
\ No newline at end of file
+}
